package name.nathanmcrae.numbersstation;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.logging.FileHandler;
import java.util.logging.Logger;
import java.util.logging.Level;
import javafx.application.Application;
import javafx.application.Platform;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Stage;

public class Main extends Application {
    private static final Logger logger = Logger.getLogger(Main.class.getName());

    private static Path statePath = null;

    public Path getStatePath() {
        if (statePath == null) {
            String stateHome = System.getenv("XDG_STATE_HOME");
            if (stateHome == null || stateHome.isEmpty() || !Paths.get(stateHome).isAbsolute()) {
                String userHome = System.getProperty("user.home");
                statePath = Paths.get(userHome, ".local", "state", "numbers-station");
            } else {
                statePath = Paths.get(stateHome, "numbers-station");
            }
        }

        return statePath;
    }

    @Override
    public void start(Stage primaryStage) throws Exception {
        setupLogger();
        Parent root = FXMLLoader.load(getClass().getResource("MainView.fxml"));
        primaryStage.setTitle("Numbers Station");
        primaryStage.setScene(new Scene(root));
        primaryStage.show();
        logger.info("Application started");
    }

    private void setupLogger() {
        try {
            Path logFile = getStatePath().resolve("main.log");

            if (!Files.exists(logFile.getParent())) {
                Files.createDirectories(logFile.getParent());
            }

            FileHandler fileHandler = new FileHandler(logFile.toString(), true);
            fileHandler.setFormatter(new CustomFormatter());
            logger.addHandler(fileHandler);
        } catch (IOException e) {
            System.out.println("Failed to set up logger: " + e.getMessage());
        }
    }

    public static void main(String[] args) {
        Thread.setDefaultUncaughtExceptionHandler((thread, throwable) -> {
            logger.log(Level.SEVERE, "Unhandled exception caught", throwable);
        });

        Platform.setImplicitExit(false);
        Thread.currentThread().setUncaughtExceptionHandler((thread, throwable) -> {
            logger.log(Level.SEVERE, "Unhandled exception in JavaFX application thread", throwable);
        });

        launch(args);
    }
}
